﻿Imports System.Drawing.Drawing2D

Public Class NeedleType
  'This class should use access methods to change and get these values, but I'm not up to snuff with that and didn't take the time
  'to look it up.  Just made them public so are directly accessable by the using program.

  'This class will hold the attributes to draw one hand, using a Graphics object passed to it.
  'It will angle the image around a point defined by hand_cx, hand_cy
  'The center_point can be offset from the 0,0 point passed in the Graphics object, using hand_rLoc, so for instance if
  'you had a small hand circling somewhere away from the center of the clock.
  'Also, if you had an image of a hand that wasn't at a convenient angle (like straight up), you coud rotate the hand using the 
  'edit form, and note the angle when it was straight up, and put it in the hand_zero_angle value, so the angle would be relative
  'to the hand_zero_angle, allowing you not to have to compensate "angle" for your image.
  Private handImage As Bitmap
  Public hand_cx, hand_cy As Single   'center of rotation within the handImage bitmap
  Public hand_rLoc As PointF          'relative position of the center on some external reference (i.e. where on the clock face the hand should go)
  Public hand_zero_angle As Single    'The image may not be oriented as we like, we can rotate it and store the bias here, of where "0" reference is.
  Public scaleFactor As Single = 1.0  'Assume symetrical scaling (only one value used for X and Y). Allows scaling hand to fit face
  Public angle As Single              'The angle the hand will be draw (relative to your hand_zero_angle setting, and original image)

  Private shadowImage As Bitmap       'If you want a shadow, will generate one in this bitmap from the image of the hand
  Public shadow_dx, shadow_dy As Single 'Offset, relative to hand_cx, hand_cy of where the shadow is drawn
  Private _shadow As Boolean
  ' Private Shared first As Boolean

  Public Sub New(ByVal bmp As Bitmap,
                 Optional ByVal Transparent As Boolean = True,
                 Optional ByVal Shadow As Boolean = True)
    ' handImage = bmp.Clone             'Use a copy of the bitmap passed since we might modify it.
    handImage = bmp
    If Transparent Then               'If the transparent flag is set
      handImage.MakeTransparent()     ' Set the background pixels (the pixels the color of pixel 0,0) to transparent
    End If
    If Shadow Then                                    'If the shadow option is selected
      Dim pixelColor As Color                         '  The slow method of doing this, but we only have to do it once
      _shadow = True                                  '  Flag that the shadow option is active (we need to draw it, and dispose of it later)
      shadowImage = handImage.Clone                   '  Clone a copy of our hand image
      For y As Integer = 0 To shadowImage.Height - 1  '  for each row of pixels in our image
        For x As Integer = 0 To shadowImage.Width - 1 '    for each column of pixels in our image
          pixelColor = shadowImage.GetPixel(x, y)     '      Get the color of the pixel
          If pixelColor.A <> 0 Then                   '      If it is not Transparent Then
            shadowImage.SetPixel(x, y, Color.FromArgb(64, pixelColor))  'Set the pixels in the shadow to 32/256ths opaque (12.5%)
            '  If first = False Then
            '    handImage.SetPixel(x, y, Color.FromArgb(255, Color.DarkGreen))
            '  End If
          End If                                                        '87.5% transparent
        Next
      Next
    End If
    '  first = True
    shadow_dx = 5  'offset the shadow 5 units down and to the right, by default
    shadow_dy = 5
    hand_cx = handImage.Width / 2   'default middle of the bitmap as the rotation point
    hand_cy = handImage.Height / 2
  End Sub

  Public Sub Render(ByVal g As Graphics)
    'The local offset will most likely be 0,0, so no offset, unless you want the hands to rotate somewhere else, relative to the 
    'center passed in the graphics object

    Dim gc As GraphicsContainer = g.BeginContainer 'we don't want to disturb the Graphics object passed, so save its context
    g.TranslateTransform(hand_rLoc.X, hand_rLoc.Y) 'Translate from the 0,0 point passed (probably center of clock) to a local offset
    g.ScaleTransform(scaleFactor, scaleFactor)     'Scale the hand larger or smaller (or not)
    If _shadow Then                                'If we want a shadow, draw it first so it is "underneath" the hand
      g.TranslateTransform(shadow_dx, shadow_dy)   '  offset down and to the right
      g.RotateTransform(angle)                     '  rotate so we are aligned with the angle the hand is pointing
      g.DrawImage(shadowImage, -hand_cx, -hand_cy) '  draw the shadow 
      g.RotateTransform(-angle)                    '  undo the rotation so our X,Y axis are horizontal,vertical
      g.TranslateTransform(-shadow_dx, -shadow_dy) '  undo the shadow offset (we're centered again)
    End If
    g.RotateTransform(angle)                       'Rotate the hand by the current angle value
    g.DrawImage(handImage, -hand_cx, -hand_cy)     'Draw the image relative to the point in the image considered the rotation point
    g.EndContainer(gc)                             'Restore the Graphics object to what it was when we came in.
  End Sub

  Protected Overrides Sub Finalize()
    handImage.Dispose()        'Clean up the bitmap object we created        
    If _shadow Then            'If we created a shadow bitmap,
      shadowImage.Dispose()    '  dispose of it
    End If
    MyBase.Finalize()
  End Sub
End Class
